/*
 *  gbe - gameboy emulator
 *  Copyright (C) 1999  Chuck Mason, Steven Fuller
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *
 *  Chuck Mason <chuckjr@sinclair.net>
 *  Steven Fuller <relnev@atdot.org>
 */

#ifdef BUILD_SVGA

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>

#include "svga.h"
#include "joypad.h"

#define GFX_BITDEPTH (8 / 8)

static unsigned long
	 gbe_window_width  = 0,
	 gbe_window_height = 0;
 
static char *gfx_buffer = (char *)NULL;

/* Internal function */
static void keyboard_handler(int code, int press)
{
	if(code == SCANCODE_ESCAPE)
		(press) ? joypad_press(GBE_O_ESCAPE) : joypad_release(GBE_O_ESCAPE);		
	else if(code == SCANCODE_ENTER)
		(press) ? joypad_press(GBE_START) : joypad_release(GBE_START);
	else if(code == SCANCODE_TAB)
		(press) ? joypad_press(GBE_SELECT) : joypad_release(GBE_SELECT);
	else if(code == SCANCODE_LEFTCONTROL)
		(press) ? joypad_press(GBE_B) : joypad_release(GBE_B);
	else if(code == SCANCODE_LEFTALT)
		(press) ? joypad_press(GBE_A) : joypad_release(GBE_A);
	else if(code == SCANCODE_CURSORBLOCKLEFT)
		(press) ? joypad_press(GBE_LEFT) : joypad_release(GBE_LEFT);
	else if(code == SCANCODE_CURSORBLOCKRIGHT)
		(press) ? joypad_press(GBE_RIGHT) : joypad_release(GBE_RIGHT);
	else if(code == SCANCODE_CURSORBLOCKUP)
		(press) ? joypad_press(GBE_UP) : joypad_release(GBE_UP);
	else if(code == SCANCODE_CURSORBLOCKDOWN)
		(press) ? joypad_press(GBE_DOWN) : joypad_release(GBE_DOWN);
}


int gbe_init_svga(char *display)
{
	vga_setmode(G320x200x256);
	if(keyboard_init() != 0) {
		printf("cannot initialize keyboard!\n");
		vga_setmode(TEXT);
		return 0;
	}
	keyboard_seteventhandler(keyboard_handler);
	
	return GFX_BITDEPTH;	
}

int gbe_close_svga()
{
	vga_setmode(TEXT);
	keyboard_close();
	return 1;
}


int gbe_create_window_svga(char *title, int width, int height)
{
	gbe_window_width = width;
	gbe_window_height = height;
	
	gfx_buffer = (char *)malloc(width * height * GFX_BITDEPTH);
	if(!gfx_buffer) {
		printf("unable to alloc buffer!\n");
		return 0;
	}
		
	return 1;
}


int gbe_poll_events_svga()
{
	keyboard_update();
	return 1;
}

int gbe_blit_svga()
{
	int x, y;
	char *mem;
	
	mem = vga_getgraphmem();

	mem += ((200 / 2) - (gbe_window_height / 2)) * 320;	
	for(y = 0; y < gbe_window_height; y++) {
		memcpy(mem + (320 / 2) - (gbe_window_width / 2), &gfx_buffer[y * gbe_window_width], gbe_window_width);
		mem += 320;
	}
	return 1;
}

int gbe_plot_svga(int x, int y, int color)
{
#if 0
	gfx_buffer[(y * (gbe_window_width * GFX_BITDEPTH)) + (x * GFX_BITDEPTH)]     = (color >> 8) & 0xFF;
	gfx_buffer[(y * (gbe_window_width * GFX_BITDEPTH)) + (x * GFX_BITDEPTH) + 1] = color & 0xFF;
#else
	gfx_buffer[(y * (gbe_window_width)) + (x)]     = color & 0xFF;
/*	gfx_buffer[(y * (gbe_window_width << 1)) + (x << 1) + 1] = (color >> 8) & 0xFF;*/
#endif	
	return 1;
}

#else

int gbe_init_svga(char *display)
{
	return 0;
}

int gbe_close_svga()
{
	return 1;
}

int gbe_create_window_svga(char *title, int width, int height)
{
	return 1;
}


int gbe_poll_events_svga()
{
	return 1;
}

int gbe_blit_svga()
{
	return 1;
}

int gbe_plot_svga(int x, int y, int color)
{
	return 1;
}

#endif
