/*
 *  gbe - gameboy emulator
 *  Copyright (C) 1999  Chuck Mason, Steven Fuller
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *
 *  Chuck Mason <chuckjr@sinclair.net>
 *  Steven Fuller <relnev@atdot.org>
 */

#ifdef BUILD_X11

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/Xatom.h>

#include "joypad.h"
#include "mem.h"
#include "regs.h"

#define GFX_BITDEPTH (16 / 8)

Window   gbe_x11_window, gbe_root_window;
Display *gbe_x11_display = NULL;
GC       gbe_window_gc;
    int  gbe_x11_screen  = 0;
static unsigned long
         gbe_x11_black_pixel = 0,
         gbe_x11_white_pixel = 0;
static unsigned long
	 gbe_window_width  = 0,
	 gbe_window_height = 0;
 
static char *gfx_buffer = (char *)NULL;

int gbe_init_x11(char *display)
{
	gbe_x11_display = XOpenDisplay(display);
	
	if(!gbe_x11_display) {
		printf("Unable to open display %s!\n", (display) ? display : "");
		return 0;
	}
	
	gbe_x11_screen = DefaultScreen(gbe_x11_display);
	gbe_root_window= RootWindow(gbe_x11_display, gbe_x11_screen);
	
	gbe_x11_black_pixel = BlackPixel(gbe_x11_display, gbe_x11_screen);
	gbe_x11_white_pixel = WhitePixel(gbe_x11_display, gbe_x11_screen);

	return GFX_BITDEPTH;	
}

int gbe_close_x11()
{
	return 1;
}

int gbe_create_window_x11(char *title, int width, int height)
{
	XSetWindowAttributes attrs;
	unsigned long attr_mask;
	Visual *visual = CopyFromParent;
	XSizeHints *sizehints;
	XGCValues gcvalues;
	
	gbe_window_width = width;
	gbe_window_height = height;

	attrs.border_pixel     = gbe_x11_white_pixel;
	attrs.background_pixel = gbe_x11_black_pixel;
	attrs.event_mask = KeyPressMask | KeyReleaseMask | ExposureMask;
	attr_mask = CWBackPixel | CWBorderPixel | CWEventMask;
	
	gbe_x11_window = XCreateWindow(gbe_x11_display, 
				       gbe_root_window,
				       10, 10,
				       width, height,
				       2,
				       CopyFromParent,
				       InputOutput,
				       visual,
				       attr_mask, &attrs);
	
	if(gbe_x11_window == (Window) None) {
		printf("Unable to create X window!\n");
		return 0;
	}

	XStoreName(gbe_x11_display, gbe_x11_window, title);
	XSetIconName(gbe_x11_display, gbe_x11_window, title);

	sizehints = XAllocSizeHints();

	if(sizehints) {
		sizehints->min_width = width;
		sizehints->min_height = height;
		sizehints->max_width = width;
		sizehints->max_height = height;
		sizehints->base_width = width;
		sizehints->base_height = height;

		sizehints->flags = PMinSize | PMaxSize | PBaseSize;

		XSetWMNormalHints(gbe_x11_display, gbe_x11_window, sizehints);
		XFree(sizehints);
	}

	gcvalues.foreground = gbe_x11_black_pixel;
	gcvalues.background = gbe_x11_white_pixel;
        gbe_window_gc = XCreateGC(gbe_x11_display, gbe_x11_window, GCForeground | GCBackground, &gcvalues);
        
	XMapRaised(gbe_x11_display, gbe_x11_window);
	XMapSubwindows(gbe_x11_display, gbe_x11_window);
		
	XFlush(gbe_x11_display);

	/* X is all good, allocate buffer */
	gfx_buffer = (char *)malloc(width * height * GFX_BITDEPTH);
	if(!gfx_buffer) {
		printf("unable to alloc buffer!\n");
		return 0;
	}
		
	return 1;
}

/* Internal function */
static void handler_keypress(void *a, void *b)
{
        KeySym key = (KeySym) b;
        if(key == XK_Escape)
		joypad_press(GBE_O_ESCAPE);
	else if(key == XK_Return) 
		joypad_press(GBE_START);
	else if(key == XK_Tab)
		joypad_press(GBE_SELECT);
	else if(key == XK_Control_L)
		joypad_press(GBE_B);
	else if(key == XK_Alt_L)
		joypad_press(GBE_A);
	else if(key == XK_Up)
		joypad_press(GBE_UP);
	else if(key == XK_Down)
		joypad_press(GBE_DOWN);
	else if(key == XK_Left)
		joypad_press(GBE_LEFT);
	else if(key == XK_Right)
		joypad_press(GBE_RIGHT);
}

/* Internal function */
static void handler_keyrelease(void *a, void *b)
{
        KeySym key = (KeySym) b;
        if(key == XK_Escape)
        	joypad_release(GBE_O_ESCAPE);
	else if(key == XK_Return) 
		joypad_release(GBE_START);
	else if(key == XK_Tab)
		joypad_release(GBE_SELECT);
	else if(key == XK_Control_L)
		joypad_release(GBE_B);
	else if(key == XK_Alt_L)
		joypad_release(GBE_A);
	else if(key == XK_Up)
		joypad_release(GBE_UP);
	else if(key == XK_Down)
		joypad_release(GBE_DOWN);
	else if(key == XK_Left)
		joypad_release(GBE_LEFT);
	else if(key == XK_Right)
		joypad_release(GBE_RIGHT);
}

int gbe_poll_events_x11()
{
	XEvent event;
	KeySym kbsym;
	
	char tmp[16];

	while(XPending(gbe_x11_display)) {
		XNextEvent(gbe_x11_display, &event);

		switch(event.type) {
			case KeyPress:
				XLookupString((XKeyEvent*) &event, tmp, 15, &kbsym, NULL);
				handler_keypress(&tmp[0], (void *)kbsym);
				break;
			case KeyRelease:
				XLookupString((XKeyEvent*) &event, tmp, 15, &kbsym, NULL);
				handler_keyrelease(&tmp[0], (void *)kbsym);
				break;
			default:
				break;
		}
	}
	return 1;
}

int gbe_blit_x11()
{
	static char i = 0;
	static	XImage *myimage;
	Visual *visual = CopyFromParent;
	
	if(i == 0) {
		myimage = XCreateImage(gbe_x11_display, visual, GFX_BITDEPTH * 8, ZPixmap, 0, 
				       gfx_buffer, 
                        	       gbe_window_width, 
	                               gbe_window_height,
        	                       GFX_BITDEPTH * 8,
                	               GFX_BITDEPTH * gbe_window_width);
	        XInitImage(myimage);
		i = 1;
	}
        
#if 1
        XPutImage(gbe_x11_display, gbe_x11_window, gbe_window_gc, myimage, 0, 0, 0, 0, gbe_window_width, gbe_window_height);
#else
	XPutImage(gbe_x11_display, gbe_x11_window, gbe_window_gc, myimage, 0, CURLINE, 0, CURLINE, gbe_window_width, 1);
#endif
	return 1;
}

int gbe_plot_x11(int x, int y, int color)
{
#if 0
	gfx_buffer[(y * (gbe_window_width * GFX_BITDEPTH)) + (x * GFX_BITDEPTH)]     = (color >> 8) & 0xFF;
	gfx_buffer[(y * (gbe_window_width * GFX_BITDEPTH)) + (x * GFX_BITDEPTH) + 1] = color & 0xFF;
#else
	gfx_buffer[(y * (gbe_window_width << 1)) + (x << 1)]     = (color >> 8) & 0xFF;
	gfx_buffer[(y * (gbe_window_width << 1)) + (x << 1) + 1] = color & 0xFF;
#endif	
	return 1;
}

#else

int gbe_init_x11(char *display)
{
	return 0;
}

int gbe_close_x11()
{
	return 1;
}

int gbe_create_window_x11(char *title, int width, int height)
{
	return 1;
}


int gbe_poll_events_x11()
{
	return 1;
}

int gbe_blit_x11()
{
	return 1;
}

int gbe_plot_x11(int x, int y, int color)
{
	return 1;
}

#endif
